function comp_CVinv
%This function is used for the numerical example from Remark 4 in Section 2
% The matrix \Gamma from the paper is denoted here by C
% Vinv is V^{-1}

close all

n = 3000;               %sample size 
Aq = [1 -1.32 0.81];    %"true" AR model
Ntr = 5000;             %no. of trials
MaxLen = 200;           %total number of samples generated
lambda = 0.995;         %forgetting factor 
Kmax = 15;              %max. order fitted AR model
m = 2*Kmax;             %param. initialization

%Initialization
L = diag(lambda.^(m-2:-1:0));

%Generate data
data = zeros(Ntr,m);
for i=1:Ntr,
    temp = filter(1,Aq,randn(1,MaxLen));
    data(i,:) = temp(end-m+1:end);
end

%Compute det(Vinv)
detVinv = zeros(Kmax,Ntr);
for p=1:Kmax,
    for i=1:Ntr,
        y = data(i,:);
        X = toeplitz([y(1); zeros(p-1,1)],y(1:m-1));
        detVinv(p,i) = det(X*L*X');
    end
end

%Compute det(C)
detC = comp_C(Aq,Kmax,0);

proc = zeros(Kmax,1);
for p=1:Kmax,
    T1 = p*log(lambda)*ones(1,Ntr);
    detCmod = detC(p)/(1-lambda)^p;
    T2 = (1-lambda-1/(n-m))*log( detCmod./detVinv(p,:) );
    T = T1+T2;
    count = length(find(T<0));
    proc(p) = 100*count/length(T);
end

figure
bar(1:Kmax,proc,'b');
xlabel('Order of the fitted AR model');
ylabel('Percentage of trials');


