function criteria_new(ex,NTRIALS)
%input:
%   ex              : example
%   NTRIALS         : no. of trials 
%-------------------------------------------------------


KMIN = 0;             %min. value for the AR order (recommended to be 0)
KMAX = 15;            %max. value for the AR order
m = 2*KMAX;           %used in the initialization of the ITC
lambda = 0.99;        %forgetting factor (must be from (0,1))
sigma_ar = 1;         %standard deviation for the driven noise
flag_mod = 0;         %modified criteria are evalauted when flag_mod=1;  
eta = [];             %will be computed if flag_mod=1  

%folder used for saving the results
folder_name_save = strcat('trials_ex', num2str(ex));
%create a folder to store the results
[s,~,~] = eval('mkdir(folder_name_save);');
if s~=1,
    fprintf('I cannot create the folder for the files with the results.\n');
    return;
end

if ex==11,      %Example 1(a) in the paper
elseif ex==12,  %Example 1(b) in the paper
    folder_name_load = strcat('trials_ex', num2str(11));
    lambda = 0.995;
elseif ex==21,  %Example 2(a) in the paper
elseif ex==22,  %Example 2(b) in the paper
elseif ex==31,  %Examples 3(a) and 3(b) in the paper
    flag_mod = 1;
elseif ex==32,  %Examples 3(c) and 3(d) in the paper
    flag_mod = 1;
    sigma_ar = 10;
end
    
for tr=1:NTRIALS, 
   %mark on the screen the current trial
    if rem(tr,100)==0,
        fprintf('.');
        if rem(tr,1000)==0,
            fprintf('\n');
        end
    end
    if tr==NTRIALS,
        fprintf('\n');
    end
    %generate/load data
    if ex==12,
        fname = strcat(folder_name_load,'/trial_',num2str(tr));
        load(fname,'y','true_order');
        clear fname
        n = length(y);
    else
        [y,n,true_order] = gen_data_nov28_13(ex,sigma_ar,m);
    end
    if n==0,
        fprintf('Invalid example.\n');
        return;
    end
    %compute $\eta_{\lambda,m}$
    if flag_mod,
        eta = comp_eta(y,KMAX,m,lambda);
    end
    %lattice predictor    
     [Ehat,Re,gamma,E,~,~,~]=lat_mar07_09(KMAX,y,lambda);
    %BIC
     kBIC = comp_BIC(KMIN,KMAX,Re,m,lambda);       
    %AIC
     kAIC = comp_AIC(KMIN,KMAX,Re,m,lambda);
    %PDC
     [kPDC,kPDCmod] = comp_PDC(KMIN,KMAX,Re,gamma,m,lambda,flag_mod,eta);    
    %SNML
     [kSNML,kSNMLmod] = comp_SNML(KMIN,KMAX,E,gamma,m,lambda,flag_mod,eta);
     %SDNML1
     [kSDNML1,kSDNML1mod] = comp_UrabeSDNML(KMIN,KMAX,Ehat,E,gamma,m,flag_mod,eta);
     %SDNML2
     [kSDNML2,kSDNML2mod]= comp_TakahashiSDNML(KMIN,KMAX,Ehat,E,gamma,m,lambda,flag_mod,eta);
     %save all variables
       fname = strcat(folder_name_save,'/trial_',num2str(tr));
       if flag_mod,
         eval('save(fname,''y'',''true_order'',''m'',''kBIC'',''kAIC'',''kPDC'',''kPDCmod'',''kSNML'',''kSNMLmod'',''kSDNML1'',''kSDNML1mod'',''kSDNML2'',''kSDNML2mod'');');
       else
         eval('save(fname,''y'',''true_order'',''m'',''kBIC'',''kAIC'',''kPDC'',''kSNML'',''kSDNML1'',''kSDNML2'');');
       end
end %trial

%------------------------------------------------------- 
% Important: equivalence with the notations from the paper.            
% For the AR order (i-1) and t=j, we have:                                                               
% Ehat(i,j)   = e_{\lambda,t}                                        
% Re(i,j)     = R_{\lambda,t}                                        
% gamma(i,j)  = 1-d_{\lambda,t} = 1/(1+c_{\lambda,t})                
%  E(i,j)      = \hat{e}_{\lambda,t}                                  
%------------------------------------------------------- 

%-------------------------------------------------------
function eta = comp_eta(y,KMAX,m,lambda)
    %We assume KMIN=0
    
    eta = zeros(KMAX+1,1); 
    L = diag(lambda.^(m-2:-1:0));
    for p=1:KMAX,
        X = toeplitz([y(1); zeros(p-1,1)],y(1:m-1));
        eta(p+1) = -log(det(X*L*X'));
    end
%-------------------------------------------------------
function kBIC = comp_BIC(KMIN,KMAX,Re,m,lambda)
    S = zeros(size(Re));
    for j=(m+1):size(S,2),
        nef = (1-lambda^j)/(1-lambda);
        S(:,j) = nef/2*log(Re(:,j)/nef) + 1/2*(1:size(S,1))'*log(nef);
    end
    S1 = S(KMIN+1:KMAX+1,:);
    [~,kBIC] = min(S1,[],1);
    kBIC = kBIC+KMIN-1;   
%-------------------------------------------------------
function kAIC = comp_AIC(KMIN,KMAX,Re,m,lambda)
    S = zeros(size(Re));
    for j=(m+1):size(S,2),
        nef = (1-lambda^j)/(1-lambda);
        S(:,j) = nef/2*log(Re(:,j)/nef) + (1:size(S,1))';
    end
    S1 = S(KMIN+1:KMAX+1,:);
    [~,kAIC] = min(S1,[],1);
    kAIC = kAIC+KMIN-1;
%------------------------------------------------------- 
function [kPDC,kPDCmod] = comp_PDC(KMIN,KMAX,Re,gamma,m,lambda,flag_mod,eta)
    %We assume KMIN=0 and m>1
    D = zeros(size(Re));
    PDC = zeros(size(Re));
    for j=(m+1):size(PDC,2),
         nef = (1-lambda^j)/(1-lambda); 
         PDC(1,j) = nef/2*log(Re(1,j)/nef);
         D(2:end,j) = D(2:end,j-1) - 1/2*log(gamma(2:end,j)./(lambda.^(1:size(PDC,1)-1)'));
         PDC(2:end,j) = nef/2*log(Re(2:end,j)/nef) + D(2:end,j);
    end  
    S1 = PDC(KMIN+1:KMAX+1,:);
    [~,kPDC] = min(S1,[],1);
    kPDC = kPDC+KMIN-1;
    if flag_mod,
        S1 = S1 - kron(ones(1,size(S1,2)),(eta/2));
        [~,kPDCmod] = min(S1,[],1);
        kPDCmod = kPDCmod+KMIN-1;
    else
        kPDCmod = [];
    end
%-------------------------------------------------------
function [kSNML,kSNMLmod] = comp_SNML(KMIN,KMAX,E,gamma,m,lambda,flag_mod,eta) 
    E = E.^2;
    D = zeros(size(E));
    F = zeros(size(E));
    SNML = zeros(size(E));    
    for j=(m+1):size(SNML,2),
        nef = (1-lambda^j)/(1-lambda); 
        F(:,j) = lambda*F(:,j-1) + E(:,j);
        SNML(1,j) = nef/2*log(F(1,j)/nef);
        D(2:end,j) = D(2:end,j-1) - log( gamma(2:end,j)./(lambda.^(1:size(SNML,1)-1)') );
        SNML(2:end,j) = nef/2*log(F(2:end,j)/nef) + D(2:end,j);
    end    
    S1 = SNML(KMIN+1:KMAX+1,:);
    [~,kSNML] = min(S1,[],1);
    kSNML = kSNML+KMIN-1;
    if flag_mod,
        S1 = S1 - kron(ones(1,size(S1,2)),eta);
        [~,kSNMLmod] = min(S1,[],1);
        kSNMLmod = kSNMLmod+KMIN-1;
    else
        kSNMLmod = [];
    end
%-------------------------------------------------------
    function [kSDNML1,kSDNML1mod] = comp_UrabeSDNML(KMIN,KMAX,Ehat,E,gamma,m,flag_mod,eta) 
    vec=log(Ehat(2:end,m+1))/2;
    clear Ehat
    E = E.^2;
    D = zeros(size(E));
    F = zeros(size(E));
    SDNML1 = zeros(size(E));
    for j=(m+1):size(SDNML1,2),
        nef = j-m; 
        F(:,j) = F(:,j-1) + E(:,j);
        SDNML1(1,j) = nef/2*log(F(1,j));
        D(2:end,j) = D(2:end,j-1) - log( gamma(2:end,j) );
        SDNML1(2:end,j) = nef/2*log(F(2:end,j)) + D(2:end,j) - vec;
    end    
    S1 = SDNML1(KMIN+1:KMAX+1,:);
    [~,kSDNML1] = min(S1,[],1);
    kSDNML1 = kSDNML1+KMIN-1;
    if flag_mod,
        S1 = S1 - kron(ones(1,size(S1,2)),eta);
        [~,kSDNML1mod] = min(S1,[],1);
        kSDNML1mod = kSDNML1mod+KMIN-1;
    else
        kSDNML1mod = [];
    end
    
%------------------------------------------------------- 
    function [kSDNML2,kSDNML2mod] = comp_TakahashiSDNML(KMIN,KMAX,Ehat,E,gamma,m,lambda,flag_mod,eta) 
    vec=log(Ehat(2:end,m+1))/2;
    clear Ehat
    E = E.^2;
    D = zeros(size(E));
    F = zeros(size(E));
    SDNML2 = zeros(size(E));     
    for j=(m+1):size(SDNML2,2),
        nef = j-m; 
        F(:,j) = lambda*F(:,j-1) + E(:,j);
        SDNML2(1,j) = nef/2*log(F(1,j));
        D(2:end,j) = D(2:end,j-1) - log( gamma(2:end,j) );
        SDNML2(2:end,j) = nef/2*log(F(2:end,j)) + D(2:end,j) - vec;
    end    
    S1 = SDNML2(KMIN+1:KMAX+1,:);
    [~,kSDNML2] = min(S1,[],1);
    kSDNML2 = kSDNML2+KMIN-1;
    if flag_mod,
        S1 = S1 - kron(ones(1,size(S1,2)),eta);
        [~,kSDNML2mod] = min(S1,[],1);
        kSDNML2mod = kSDNML2mod+KMIN-1;
    else
        kSDNML2mod = [];
    end
    
    
    