function [y,n,true_order] = gen_data_nov28_13(ex,sigma_ar,m)
%input:
%   ex              : which example
%   sigma_ar        : standard deviation for the driven noise
%   m               : used in the initialization of the ITC
%-------------------------------------------------------
%ex : 11 for example 1(a)
%   : 21 for example 2(a)
%   : 22 for example 2(b)
%   : 31 for examples 3(a) and 3(b)
%   : 32 for examples 3(c) and 3(d)
%-------------------------------------------------------

if ex==11,
    [y,n,true_order] = gen_data_1(sigma_ar);
elseif ex==21,
    [y,n,true_order] = gen_data_21(sigma_ar);
elseif ex==22,
    [y,n,true_order] = gen_data_22(sigma_ar,m);
elseif (ex==31) || (ex==32),
    [y,n,true_order] = gen_data_3(sigma_ar);
else
    y = [];
    n = 0;
    true_order = [];
end
%-------------------------------------------------------
function [y,n,true_order] = gen_data_1(sigma_ar)
% sigma_ar  =   standard deviation for the driven noise
%example from Hannan et al. (1989)
true_order = [zeros(1,1000) 6*ones(1,1000) 8*ones(1,1000) zeros(1,1000)];
A8 = [1 -0.9896 0.8097 -0.8912 0.6736 -0.7575 0.5850 -0.6077 0.5220]; %Hannan table 3
A6 = [1 -0.4397 -0.1316 0.0905 -0.1053 -0.2814 0.5120]; %Hannan table 3
n = 4000; %no. of samples

y = zeros(n,1);
eps = sigma_ar*randn(n,1);
%segment no. 1
y(1:1000) = eps(1:1000);
%segment no. 2
for i=1001:2000,
    y(i) = -A6(2:end)*y(i-1:-1:i-6) + eps(i);
end
%segment no. 3
for i=2001:3000,
    y(i) = -A8(2:end)*y(i-1:-1:i-8) + eps(i);
end
%segment no. 4
y(3001:4000) = eps(3001:4000);
%-------------------------------------------------------
function [y,n,true_order] = gen_data_21(sigma_ar)
% sigma_ar  =   standard deviation for the driven noise
%example from Hannan et al. (1989) (order of segments is changed)
true_order = [ 8*ones(1,1000) zeros(1,1000) 6*ones(1,1000) zeros(1,1000)];
A8 = [1 -0.9896 0.8097 -0.8912 0.6736 -0.7575 0.5850 -0.6077 0.5220]; %Hannan table 3
A6 = [1 -0.4397 -0.1316 0.0905 -0.1053 -0.2814 0.5120]; %Hannan table 3
n = 4000; %no. of samples

y = zeros(n,1);
eps = sigma_ar*randn(n,1);
%segment no. 1
y1 = zeros(1008,1);
for i=9:1008,
    y1(i) = -A8(2:end)*y1(i-1:-1:i-8) + eps(i-8);
end
y(1:1000) = y1(9:1008);
%segment no. 2
y(1001:2000) = eps(1001:2000);
%segment no. 3
for i=2001:3000,
    y(i) = -A6(2:end)*y(i-1:-1:i-6) + eps(i);
end
%segment no. 4
y(3001:4000) = eps(3001:4000);
%------------------------------------------------------- 
function [y,n,true_order] = gen_data_22(sigma_ar,m)
% sigma_ar  =   standard deviation for the driven noise
%example from Hann an et al. (1989) (order of segments is changed)
%initialization is changed
true_order = [zeros(1,m) 8*ones(1,1000-m) zeros(1,1000) 6*ones(1,1000) zeros(1,1000)];
A8 = [1 -0.9896 0.8097 -0.8912 0.6736 -0.7575 0.5850 -0.6077 0.5220]; %Hannan table 3
A6 = [1 -0.4397 -0.1316 0.0905 -0.1053 -0.2814 0.5120]; %Hannan table 3
n = 4000; %no. of samples

y = zeros(n,1);
eps = sigma_ar*randn(n,1);
%segment no. 1
y(1:m) = eps(1:m);
%segment no. 2
for i=(m+1):1000,
    y(i) = -A8(2:end)*y(i-1:-1:i-8) + eps(i);
end
%segment no. 3
y(1001:2000) = eps(1001:2000);
%segment no. 4
for i=2001:3000,
    y(i) = -A6(2:end)*y(i-1:-1:i-6) + eps(i);
end
%segment no. 5
y(3001:4000) = eps(3001:4000);
%-------------------------------------------------------
function [y,n,true_order] = gen_data_3(sigma_ar)
% sigma_ar  =   standard deviation for the driven noise
true_order = [ 8*ones(1,1000) 1*ones(1,3000)];
A8 = [1 -0.9896 0.8097 -0.8912 0.6736 -0.7575 0.5850 -0.6077 0.5220]; %Hannan table 3
n = 4000; %no. of samples

y = zeros(n,1);
eps = sigma_ar*randn(n,1);
%segment no. 1
y1 = zeros(1008,1);
for i=9:1008,
    y1(i) = -A8(2:end)*y1(i-1:-1:i-8) + eps(i-8);
end
y(1:1000) = y1(9:1008);
%segment no. 2
for i=1001:4000,
    y(i) = -0.5*y(i-1) + eps(i);
end
